@@ -33,7 +33,7 @@ import java.io.ObjectInput;
  * @author Eric D. Friedman
  * @author Rob Eden (auto-compaction)
  *
- * @version $Id: THash.java,v 1.13 2008/05/07 21:25:45 robeden Exp $
+ * @version $Id: THash.java,v 1.14 2008/10/08 16:39:10 robeden Exp $
  */
 
 abstract public class THash implements Cloneable, Externalizable {
@@ -125,7 +125,7 @@ abstract public class THash implements Cloneable, Externalizable {
         // found to be a pretty good starting auto-compaction factor.
         _autoCompactionFactor = loadFactor;
         
-        setUp((int)Math.ceil(initialCapacity / loadFactor));
+        setUp(HashFunctions.fastCeil(initialCapacity / loadFactor));
     }
 
     public Object clone() {
@@ -169,7 +169,7 @@ abstract public class THash implements Cloneable, Externalizable {
      */
     public void ensureCapacity(int desiredCapacity) {
         if (desiredCapacity > (_maxSize - size())) {
-            rehash(PrimeFinder.nextPrime((int) Math.ceil(
+            rehash(PrimeFinder.nextPrime(HashFunctions.fastCeil(
                 (desiredCapacity + size()) / _loadFactor) + 1));
             computeMaxSize(capacity());
         }
@@ -194,7 +194,7 @@ abstract public class THash implements Cloneable, Externalizable {
      */
     public void compact() {
         // need at least one free spot for open addressing
-        rehash(PrimeFinder.nextPrime((int)Math.ceil(size()/_loadFactor) + 1));
+        rehash(PrimeFinder.nextPrime(HashFunctions.fastCeil(size()/_loadFactor) + 1));
         computeMaxSize(capacity());
 
         // If auto-compaction is enabled, re-determine the compaction interval
@@ -332,8 +332,7 @@ abstract public class THash implements Cloneable, Externalizable {
      */
     private void computeMaxSize(int capacity) {
         // need at least one free slot for open addressing
-        _maxSize = Math.min(capacity - 1,
-                            (int)Math.floor(capacity * _loadFactor));
+        _maxSize = Math.min(capacity - 1, (int) (capacity * _loadFactor));
         _free = capacity - _size; // reset the free element count
     }
 
@@ -344,7 +343,10 @@ abstract public class THash implements Cloneable, Externalizable {
      */
     private void computeNextAutoCompactionAmount( int size ) {
         if ( _autoCompactionFactor != 0 ) {
-            _autoCompactRemovesRemaining = Math.round( size * _autoCompactionFactor );
+            // NOTE: doing the round ourselves has been found to be faster than using
+            //       Math.round.
+            _autoCompactRemovesRemaining =
+                ( int ) ( ( size * _autoCompactionFactor ) + 0.5f );
         }
     }
 
