@@ -28,7 +28,7 @@ import java.util.*;
  * Created: Sun Nov  4 08:52:45 2001
  *
  * @author Eric D. Friedman
- * @version $Id: THashMap.java,v 1.28 2008/04/04 18:51:04 robeden Exp $
+ * @version $Id: THashMap.java,v 1.33 2008/05/08 17:42:55 robeden Exp $
  */
 public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externalizable {
     static final long serialVersionUID = 1L;
@@ -143,6 +143,7 @@ public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externali
         int capacity;
         
         capacity = super.setUp(initialCapacity);
+        //noinspection unchecked
         _values = (V[]) new Object[capacity];
         return capacity;
     }
@@ -153,17 +154,37 @@ public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externali
      * @param key an <code>Object</code> value
      * @param value an <code>Object</code> value
      * @return the previous value associated with <tt>key</tt>,
-     * or null if none was found.
+     * or {@code null} if none was found.
      */
     public V put(K key, V value) {
+        int index = insertionIndex(key);
+        return doPut(key, value, index);
+    }
+    
+    /**
+     * Inserts a key/value pair into the map if the specified key is not already
+     * associated with a value.
+     * 
+     * @param key an <code>Object</code> value
+     * @param value an <code>Object</code> value
+     * @return the previous value associated with <tt>key</tt>,
+     * or {@code null} if none was found.
+     */
+    public V putIfAbsent(K key, V value) {
+        int index = insertionIndex(key);
+        if (index < 0)
+            return _values[-index - 1];
+        return doPut(key, value, index);
+    }
+    
+    private V doPut(K key, V value, int index) {
         V previous = null;
         Object oldKey;
-        int index = insertionIndex(key);
-    boolean isNewMapping = true;
+        boolean isNewMapping = true;
         if (index < 0) {
             index = -index -1;
             previous = _values[index];
-        isNewMapping = false;
+            isNewMapping = false;
         }
         oldKey = _set[index];
         _set[index] = key;
@@ -200,10 +221,10 @@ public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externali
     }
 
     public String toString() {
-        final StringBuffer buf = new StringBuffer("{");
-        forEachEntry(new TObjectObjectProcedure() {
+        final StringBuilder buf = new StringBuilder("{");
+        forEachEntry(new TObjectObjectProcedure<K,V>() {
             private boolean first = true;
-            public boolean execute(Object key, Object value) {
+            public boolean execute(K key, V value) {
                 if ( first ) first = false;
                 else buf.append( "," );
 
@@ -758,7 +779,10 @@ public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externali
 
     public void writeExternal( ObjectOutput out ) throws IOException {
         // VERSION
-        out.writeByte( 0 );
+        out.writeByte( 1 );
+
+        // NOTE: Super was not written in version 0
+        super.writeExternal( out );
 
         // NUMBER OF ENTRIES
         out.writeInt( _size );
@@ -774,7 +798,10 @@ public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externali
         throws IOException, ClassNotFoundException {
 
         // VERSION
-        in.readByte();
+        byte version = in.readByte();
+
+        // NOTE: super was not written in version 0
+        if ( version != 0 ) super.readExternal( in );
 
         // NUMBER OF ENTRIES
         int size = in.readInt();
@@ -782,7 +809,9 @@ public class THashMap<K,V> extends TObjectHash<K> implements Map<K,V>, Externali
 
         // ENTRIES
         while (size-- > 0) {
+            //noinspection unchecked
             K key = (K) in.readObject();
+            //noinspection unchecked
             V val = (V) in.readObject();
             put(key, val);
         }
