@@ -32,7 +32,7 @@ import java.lang.reflect.Array;
  * Created: Sat Nov  3 10:38:17 2001
  *
  * @author Eric D. Friedman
- * @version $Id: THashSet.java,v 1.20 2008/05/07 19:26:30 robeden Exp $
+ * @version $Id: THashSet.java,v 1.21 2008/10/07 20:33:56 robeden Exp $
  */
 
 public class THashSet<E> extends TObjectHash<E>
@@ -374,7 +374,10 @@ public class THashSet<E> extends TObjectHash<E>
 
     public void writeExternal( ObjectOutput out ) throws IOException {
         // VERSION
-        out.writeByte( 0 );
+        out.writeByte( 1 );
+
+        // NOTE: Super was not written in version 0
+        super.writeExternal( out );
 
         // NUMBER OF ENTRIES
         out.writeInt( _size );
@@ -390,7 +393,10 @@ public class THashSet<E> extends TObjectHash<E>
         throws IOException, ClassNotFoundException {
 
         // VERSION
-        in.readByte();
+        byte version = in.readByte();
+
+        // NOTE: super was not written in version 0
+        if ( version != 0 ) super.readExternal( in );
 
         // NUMBER OF ENTRIES
         int size = in.readInt();
