@@ -51,7 +51,7 @@ import java.util.NoSuchElementException;
  * </p>
  *
  * @author Eric D. Friedman
- * @version $Id: TLinkedList.java,v 1.13 2008/05/05 22:49:19 robeden Exp $
+ * @version $Id: TLinkedList.java,v 1.15 2009/03/31 19:43:14 robeden Exp $
  * @see gnu.trove.TLinkable
  */
 
@@ -319,6 +319,9 @@ public class TLinkedList<T extends TLinkable> extends AbstractSequentialList<T>
      */
     public T removeFirst() {
         T o = _head;
+
+        if ( o == null ) return null;
+
         T n = (T) o.getNext();
         o.setNext(null);
 
@@ -340,6 +343,9 @@ public class TLinkedList<T extends TLinkable> extends AbstractSequentialList<T>
      */
     public T removeLast() {
         T o = _tail;
+
+        if ( o == null ) return null;
+
         T prev = (T) o.getPrevious();
         o.setPrevious(null);
 
@@ -402,7 +408,12 @@ public class TLinkedList<T extends TLinkable> extends AbstractSequentialList<T>
             p = (T) link.getPrevious();
             n = (T) link.getNext();
 
-            if (n == null && p == null) { // emptying the list
+            if (n == null && p == null ) { // emptying the list
+                // It's possible this object is not something that's in the list. So,
+                // make sure it's the head if it doesn't point to anything. This solves
+                // problems caused by removing something multiple times.
+                if (o != _head) return false;
+
                 _head = _tail = null;
             } else if (n == null) { // this is the tail
                 // make previous the new tail
@@ -509,7 +520,7 @@ public class TLinkedList<T extends TLinkable> extends AbstractSequentialList<T>
         out.writeObject(_head);
 
         // TAIL
-        out.writeObject(_head);
+        out.writeObject(_tail);
     }
 
     public void readExternal( ObjectInput in )
