@@ -24,15 +24,15 @@ package gnu.trove;
  * Created: Mon Nov  5 22:19:36 2001
  *
  * @author Eric D. Friedman
- * @version $Id: TObjectFunction.java,v 1.2 2001/11/10 22:30:28 ericdf Exp $
+ * @version $Id: TObjectFunction.java,v 1.3 2006/11/10 23:27:56 robeden Exp $
  */
 
-public interface TObjectFunction {
+public interface TObjectFunction<T,R> {
     /**
      * Execute this function with <tt>value</tt>
      *
      * @param value an <code>Object</code> input
      * @return an <code>Object</code> result
      */
-    public Object execute(Object value);
+    public R execute(T value);
 }// TObjectFunction
