@@ -24,19 +24,19 @@ package gnu.trove;
  * Created: Wed Nov 28 21:30:53 2001
  *
  * @author Eric D. Friedman
- * @version $Id: TObjectHashIterator.java,v 1.1 2001/12/28 20:04:39 ericdf Exp $
+ * @version $Id: TObjectHashIterator.java,v 1.2 2006/11/10 23:27:56 robeden Exp $
  */
 
-class TObjectHashIterator extends THashIterator  {
-    protected final TObjectHash _objectHash;
+class TObjectHashIterator<E> extends THashIterator<E>  {
+    protected final TObjectHash<E> _objectHash;
 
-    public TObjectHashIterator(TObjectHash hash) {
+    public TObjectHashIterator(TObjectHash<E> hash) {
         super(hash);
         _objectHash = hash;
     }
 
-    protected Object objectAtIndex(int index) {
-        return _objectHash._set[index];
+    protected E objectAtIndex(int index) {
+        return (E)_objectHash._set[index];
     }
     
 } // TObjectHashIterator
