@@ -23,7 +23,7 @@ package gnu.trove.benchmark;
  * Created: Thu Nov 22 20:47:16 2001
  *
  * @author Eric D. Friedman
- * @version $Id: Result.java,v 1.2 2006/11/10 23:27:59 robeden Exp $
+ * @version $Id: Result.java,v 1.3 2008/05/07 19:26:31 robeden Exp $
  */
 class Result {
     long theirs;
@@ -122,7 +122,7 @@ class Result {
     }
 
     public String toString() {
-        StringBuffer b = new StringBuffer();
+        StringBuilder b = new StringBuilder();
         b.append(getDescription() + "\n");
         b.append("Iterations: " + getIterations() + "\n");
         b.append("Their total (msec): " + getTheirs() + "\n");
