@@ -325,4 +325,29 @@ public class O2PHashMapTest extends TestCase {
         assertEquals( 0, map.putIfAbsent( "Nine", 9 ) );
         assertEquals( 9, map.get( "Nine" ) );
     }
+
+  public void testPutAll() throws Exception {
+    TObjectIntHashMap<String> t = new TObjectIntHashMap<String>();
+    TObjectIntHashMap<String> m = new TObjectIntHashMap<String>();
+    m.put("one", 2);
+    m.put("two", 4);
+    m.put("three", 6);
+
+    t.put("four", 5);
+    assertEquals(1, t.size());
+
+    t.putAll(m);
+    assertEquals(4, t.size());
+    assertEquals(4, t.get("two"));
+}
+
+  public void testToString() {
+    TObjectIntHashMap<String> m = new TObjectIntHashMap<String>();
+    m.put("One", 11);
+    m.put("Two", 22);
+
+    String to_string = m.toString();
+    assertTrue(to_string, to_string.equals("{One=11,Two=22}")
+        || to_string.equals("{Two=22,One=11}"));
+  }
 }
