@@ -1,7 +1,21 @@
-/*
- * Copyright(c) 2006, NEXVU Technologies
- * All rights reserved.
- */
+///////////////////////////////////////////////////////////////////////////////
+// Copyright (c) 2006-2008, Rob Eden All Rights Reserved.
+//
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+//
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+///////////////////////////////////////////////////////////////////////////////
+
 package gnu.trove;
 
 import gnu.trove.decorator.TByteIntHashMapDecorator;
@@ -10,6 +24,7 @@ import junit.framework.TestCase;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
+import java.util.Arrays;
 
 
 /**
@@ -242,4 +257,54 @@ public class P2PHashMapTest extends TestCase {
         assertEquals( 0, map.putIfAbsent( 9, 90 ) );
         assertEquals( 90, map.get( 9 ) );
     }
+
+
+    public void testBug2037709() {
+        TIntIntHashMap m = new TIntIntHashMap();
+        for (int i = 0; i < 10; i++) {
+            m.put(i, i);
+        }
+
+        int sz = m.size();
+        assertEquals(10, sz);
+
+        int[] keys = new int[sz];
+        m.keys(keys);
+
+        boolean[] seen = new boolean[sz];
+        Arrays.fill( seen, false );
+        for (int i = 0; i < 10; i++) {
+            seen[ keys[ i ] ] = true;
+        }
+
+        for (int i = 0; i < 10; i++) {
+            if (!seen[ i ]) {
+                TestCase.fail("Missing key for: " + i);
+            }
+        }
+    }
+
+  public void testPutAll() throws Exception {
+    TIntIntHashMap t = new TIntIntHashMap();
+    TIntIntHashMap m = new TIntIntHashMap();
+    m.put(1, 2);
+    m.put(2, 4);
+    m.put(3, 6);
+
+    t.put(4, 5);
+    assertEquals(1, t.size());
+
+    t.putAll(m);
+    assertEquals(4, t.size());
+    assertEquals(4, t.get(2));
+}
+
+  public void testToString() {
+    TIntIntHashMap m = new TIntIntHashMap();
+    m.put(11, 1);
+    m.put(22, 2);
+
+    String to_string = m.toString();
+    assertTrue(to_string, to_string.equals("{11=1,22=2}") || to_string.equals("{22=2,11=1}"));
+  }
 }
