@@ -31,7 +31,7 @@ import java.util.*;
  * Created: Sat Nov  3 10:31:38 2001
  *
  * @author Eric D. Friedman
- * @version $Id: THashMapTest.java,v 1.11 2008/04/04 16:31:12 robeden Exp $
+ * @version $Id: THashMapTest.java,v 1.12 2008/05/07 19:38:11 robeden Exp $
  */
 
 public class THashMapTest extends TestCase {
@@ -61,6 +61,18 @@ public class THashMapTest extends TestCase {
         assertEquals("put/get failed", "two", map.get("One"));
         assertEquals("second put failed","two",map.put("One","foo"));
     }
+    
+    public void testPutIfAbsent() throws Exception {
+        assertEquals("putIfAbsent succeeded",null, map.putIfAbsent("One","two"));
+        assertEquals("size did not reflect putIfAbsent", 1, map.size());
+        assertEquals("putIfAbsent/get failed", "two", map.get("One"));
+        assertEquals("second putIfAbsent failed","two", map.putIfAbsent("One","foo"));
+        assertEquals("size did not reflect putIfAbsent", 1, map.size());
+        assertEquals("putIfAbsent/get failed", "two", map.get("One"));
+        assertEquals("third putIfAbsent failed", null, map.putIfAbsent("Two","bar"));
+        assertEquals("size did not reflect putIfAbsent", 2, map.size());
+        assertEquals("putIfAbsent/get failed", "bar", map.get("Two"));
+    }
 
     public void testClear() throws Exception {
         assertEquals("initial size was not zero", 0, map.size());
