@@ -27,7 +27,7 @@ import java.util.*;
  * Created: Sat Nov 10 15:57:07 2001
  *
  * @author Eric D. Friedman
- * @version $Id: TLinkedListTest.java,v 1.6 2008/05/05 22:49:19 robeden Exp $
+ * @version $Id: TLinkedListTest.java,v 1.7 2009/03/31 19:43:14 robeden Exp $
  */
 
 public class TLinkedListTest extends TestCase {
@@ -491,6 +491,79 @@ public class TLinkedListTest extends TestCase {
         assertEquals( 0, value );
     }
 
+    public void testMultipleRemove() {
+        Data one = new Data( 1 );
+        Data two = new Data( 2 );
+        Data three = new Data( 3 );
+
+        list.add( one );
+        list.add( two );
+        list.add( three );
+
+        list.remove( one );
+
+        assertEquals( 2, list.size() );
+        assertEquals( new Data( 2 ), list.get( 0 ) );
+        assertEquals( new Data( 3 ), list.get( 1 ) );
+
+        list.remove( one );
+
+        assertEquals( 2, list.size() );
+        assertEquals( new Data( 2 ), list.get( 0 ) );
+        assertEquals( new Data( 3 ), list.get( 1 ) );
+
+        list.remove( one );
+
+        assertEquals( 2, list.size() );
+        assertEquals( new Data( 2 ), list.get( 0 ) );
+        assertEquals( new Data( 3 ), list.get( 1 ) );
+
+        list.remove( one );
+
+        assertEquals( 2, list.size() );
+        assertEquals( new Data( 2 ), list.get( 0 ) );
+        assertEquals( new Data( 3 ), list.get( 1 ) );
+
+        list.remove( one );
+
+        assertEquals( 2, list.size() );
+        assertEquals( new Data( 2 ), list.get( 0 ) );
+        assertEquals( new Data( 3 ), list.get( 1 ) );
+    }
+
+
+    public void testRemoveFirstAll() {
+        list.add( new Data( 1 ) );
+        list.add( new Data( 2 ) );
+        list.add( new Data( 3 ) );
+        list.add( new Data( 4 ) );
+
+        int expected = 1;
+        Data data;
+        while( ( data = list.removeFirst() ) != null ) {
+            assertEquals( expected, data._val );
+            expected++;
+        }
+        assertTrue( list.isEmpty() );
+    }
+
+
+    public void testRemoveLastAll() {
+        list.add( new Data( 1 ) );
+        list.add( new Data( 2 ) );
+        list.add( new Data( 3 ) );
+        list.add( new Data( 4 ) );
+
+        int expected = 4;
+        Data data;
+        while( ( data = list.removeLast() ) != null ) {
+            assertEquals( expected, data._val );
+            expected--;
+        }
+        assertTrue( list.isEmpty() );
+    }
+
+
     public void testPastIndexGet() {
         try {
             list.get( 0 );
